/*
Copyright (c) 1997-2005, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.tmf;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import java.io.IOException;
import java.io.PrintWriter;

public final class ThroughputMonitorResponseWrapper extends HttpServletResponseWrapper
    {
    private ServletOutputStream stream;
    private PrintWriter writer;
    private ThroughputMonitorFilterBase filter;
    private HttpServletResponse origResponse;

    public ThroughputMonitorResponseWrapper(final ThroughputMonitorFilterBase f,
                                            final HttpServletResponse response)
        {
        super(response);
        filter = f;
        origResponse = response;
        }

    private ServletOutputStream createOutputStream()
        throws IOException
        { return(new ThroughputMonitorResponseStream(filter, origResponse)); }

    public ServletOutputStream getOutputStream()
        throws IOException
        {
        if(writer != null) { throw new IllegalStateException("getWriter() has already been called for this response"); }
        if(stream == null) { stream = createOutputStream(); }
        return(stream);
        }

    public PrintWriter getWriter()
        throws IOException
        {
        if(writer != null) { return(writer); }
        if(stream != null) { throw new IllegalStateException("getOutputStream() has already been called for this response"); }
        stream = createOutputStream();
        writer = new PrintWriter(stream);
        return(writer);
        }

    /**Intercept flushBuffer() to note that we are I/O-bound during it. */
    public void flushBuffer()
        throws IOException
        {
        final boolean didDecrement = filter.adjustBusyConnectionCount(false);
        try { super.flushBuffer(); }
        finally { if(didDecrement) { filter.adjustBusyConnectionCount(true); } }
        }

    /**Finish the response, closing output streams.
     * Note that we will probably be I/O-bound during the close()
     * if this implies pending data to be flushed first.
     */
    public void finishResponse()
        throws IOException
        {
        final boolean didDecrement = filter.adjustBusyConnectionCount(false);
        try
            {
            // Now actually close the output.
            if(writer != null)
                { writer.close(); }
            else if(stream != null)
                { stream.close(); }
            }
        finally { if(didDecrement) { filter.adjustBusyConnectionCount(true); } }
        }
    }
